package com.ec.mutualistapichincha.listasnegras.dao;

import java.util.Date;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Stateless;

import com.ec.mutualistapichincha.base.util.Ctes;
import com.ec.mutualistapichincha.listasnegras.model.ClConsepValida;
import com.ec.mutualistapichincha.listasnegras.model.ClPeps;
import com.ec.mutualistapichincha.listasnegras.model.ExcepcionesBasesNeg;
import com.ec.mutualistapichincha.listasnegras.util.MensajesValidacion;

@Stateless
public class ClPepsDaoBean extends JPADao<Integer, ClPeps> {

	@EJB
	ClConsepValidaDaoBean servClConsep;
	@EJB
	ExcepcionesBasesNegDaoBean servExcep;

	public int deleteAll() {
		return em.createQuery(" delete from ClPeps o").executeUpdate();
	}

	public ClPeps buscarCedula(final String cedula) {
		try {
			return (ClPeps) em
					.createQuery(
							"select o from ClPeps o where o.cedula = :cedula and o.estado = 'A'")
					.setParameter("cedula", cedula).getSingleResult();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public ClPeps buscarApellido(final String nombres, final String apellidos) {
		try {
			String nom = nombres + " " + apellidos;
			String sql = "select o from ClPeps o where upper(o.nombres) = :nombres and o.estado = 'A'";
			List<ClPeps> list = em.createQuery(sql, ClPeps.class)
					.setParameter("nombres", nom.trim().toUpperCase())
					.getResultList();
			if (list.size() > 0)
				return list.get(0);
			return null;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/***
	 * 
	 * @param nombre
	 * @param apellido
	 * @param cedula
	 * @return [0]( 0 No se encuentra registrado, 1 encontrado por cedula, 2
	 *         encontrado por Nombre Apellido) , [1] Cargo
	 */
	/***
	 * 
	 * @param nombre
	 * @param apellido
	 * @param cedula
	 * @return [0]( 0 No se encuentra registrado, 1 encontrado por cedula, 2
	 *         encontrado por Nombre Apellido, 3 Excepcionado) , [1] Cargo
	 */
	@SuppressWarnings("unchecked")
	public Object[] listaPEPNomb(final String cedula, String nombres,
			String apellidos, final Integer operador, final String accion,
			final String modulo) {

		if (apellidos == null)
			apellidos = "";
		if (nombres == null)
			nombres = "";

		apellidos = apellidos.trim();
		nombres = nombres.trim();

		String nom = "";
		if (nombres.equalsIgnoreCase(""))
			nom = apellidos;
		else
			nom = apellidos + " " + nombres;

		nom = nom.trim();
		String sql = "";

		sql = "select o from ClPeps o where upper(o.cedula) = :ced and o.estado = 'A'";
		List<ClPeps> n = em.createQuery(sql)
				.setParameter("ced", cedula.trim().toUpperCase())
				.getResultList();
		String cargo = "";
		if (n.size() > 0) {
			ClConsepValida consep = new ClConsepValida();
			consep.setCedula(cedula);
			if (n.size() == 1) {
				ClPeps p = n.get(0);
				String nomA = Ctes.truncarString(p.getNombres(), 90);
				consep.setNombres(nomA);
				consep.setApellidos(nomA);
				cargo = p.getCargo();
			} else {
				consep.setNombres(nom);
				consep.setApellidos(nom);
			}
			consep.setBase("PEP");
			consep.setExiste("SI");
			consep.setFecha(new Date());
			consep.setOperador(operador);
			consep.setAccion(accion);
			consep.setModulo(modulo);
			consep.setCriterioExiste("IDENTIFICACION");

			if (excepcionPep(cedula, nombres, apellidos, operador, accion,
					modulo))
				return new Object[] { 3, cargo };
			else {
				servClConsep.crearTransactionNew(consep);
				return new Object[] { 1, cargo };
			}
		}

		if (!(Ctes.stringNoNuloNoVacio(nom))) {
			ClConsepValida consep = new ClConsepValida();
			consep.setCedula(cedula);
			consep.setApellidos(apellidos);
			consep.setNombres(nombres);
			consep.setBase("PEP");
			consep.setExiste("NO");
			consep.setFecha(new Date());
			consep.setOperador(operador);
			consep.setAccion(accion);
			consep.setModulo(modulo);
			consep.setCriterioExiste("");
			servClConsep.crearTransactionNew(consep);
			return new Object[] { 0, "" };
		}

		sql = "select o from ClPeps o where upper(o.nombres) = :nombres and o.estado = 'A'";
		n = em.createQuery(sql)
				.setParameter("nombres", nom.trim().toUpperCase())
				.getResultList();

		if (n.size() > 0) {
			ClConsepValida consep = new ClConsepValida();
			consep.setApellidos(apellidos);
			consep.setNombres(nombres);
			if (n.size() == 1) {
				String c = n.get(0).getCedula();
				c = c == null ? "" : c.trim();
				consep.setCedula(c);
				cargo = n.get(0).getCargo();
			} else {
				consep.setCedula(cedula);
			}
			consep.setBase("PEP");
			consep.setExiste("SI");
			consep.setFecha(new Date());
			consep.setOperador(operador);
			consep.setAccion(accion);
			consep.setModulo(modulo);
			consep.setCriterioExiste("APELLIDOS Y NOMBRES");

			if (excepcionPep(cedula, nombres, apellidos, operador, accion,
					modulo))
				return new Object[] { 3, cargo };
			else {
				servClConsep.crearTransactionNew(consep);
				return new Object[] { 2, cargo };
			}
		} else {
			ClConsepValida consep = new ClConsepValida();
			consep.setCedula(cedula);
			consep.setApellidos(apellidos);
			consep.setNombres(nombres);
			consep.setBase("PEP");
			consep.setExiste("NO");
			consep.setFecha(new Date());
			consep.setOperador(operador);
			consep.setAccion(accion);
			consep.setModulo(modulo);
			consep.setCriterioExiste("");
			servClConsep.crearTransactionNew(consep);
			return new Object[] { 0, "" };
		}
	}

	public int totalRegistros() {
		try {
			Number num = (Number) em.createQuery(
					"select count(*) from ClPeps o").getSingleResult();
			if (num != null)
				return num.intValue();
			return 0;
		} catch (Throwable e) {
			e.printStackTrace();
			return 0;
		}
	}

	private boolean excepcionPep(final String cedula, final String nombres,
			final String apellidos, final Integer operador,
			final String accion, final String modulo) {
		ExcepcionesBasesNeg base = servExcep.buscarCedula(cedula,
				MensajesValidacion.BASEPEP);
		if (base != null) {
			ClConsepValida consep = new ClConsepValida();
			consep.setCedula(cedula);
			consep.setApellidos(apellidos);
			consep.setNombres(nombres);
			consep.setBase("PEP");
			consep.setExiste("NO");
			consep.setFecha(new Date());
			consep.setOperador(operador);
			consep.setAccion(accion);
			consep.setModulo(modulo);
			consep.setCriterioExiste("EXCEPCIONADO");
			servClConsep.crearTransactionNew(consep);
			return true;
		}
		return false;
	}
}