package com.ec.mutualistapichincha.listasnegras.dao;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.annotation.PostConstruct;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import com.ec.mutualistapichincha.listasnegras.model.EntidadBase;


@SuppressWarnings("unchecked")
public class JPADao<K extends Serializable, E extends EntidadBase> implements Dao<K, E> {

	private Class<E> entityClass;

	@PersistenceContext
	protected EntityManager em;

	@PostConstruct
	public void init() {
		ParameterizedType type = (ParameterizedType) this.getClass().getGenericSuperclass();
		entityClass = (Class<E>) type.getActualTypeArguments()[1];
	}

	@Override
	public E crear(final E entity) {
		em.persist(entity);
		return entity;
	}

	@Override
	public E actualizar(final E entity) {
		em.merge(entity);
		return entity;
	}

	@Override
	public void eliminar(final E entity) throws PersistenceException {
		em.remove(entity);
	}

	@Override
	public void eliminar(final K id) {
		E entity = buscarPorId(id);
		em.remove(entity);
	}

	@Override
	public E buscarPorId(final K id) {
		return em.find(entityClass, id);
	}

	@Override
	public List<E> buscarTodo() {
		return em.createQuery(initiateEjbql()).getResultList();
	}

	@Override
	public List<E> buscarTodo(final String... orderFields) {
		StringBuilder orderedEjbql = new StringBuilder();
		orderedEjbql.append(initiateEjbql());
		if (orderFields != null && orderFields.length != 0) {
			orderedEjbql.append(" order by ");
			for (int i = 0; i < orderFields.length; i++) {
				orderedEjbql.append(orderFields[i]);
				if (i < orderFields.length - 1)
					orderedEjbql.append(",");
			}
		}
		return em.createQuery(orderedEjbql.toString()).getResultList();
	}

	@Override
	public <O extends Object> List<E> executeQuery(final String ejbql, final Map<String, O> parameterValues) {
		Query query = em.createQuery(ejbql);

		for (Entry<String, O> entry : parameterValues.entrySet()) {
			query.setParameter(entry.getKey(), entry.getValue());
		}

		return query.getResultList();
	}

	@Override
	public void desconectar(final E entity) {
		em.detach(entity);
	}

	// private methods

	private String initiateEjbql() {
		StringBuilder ejbql = new StringBuilder("from ");
		ejbql.append(entityClass.getSimpleName());
		return ejbql.toString();
	}

	@Override
	public void immediateCommit() {
		em.flush();
	}

	protected String valorUpper(final String valor) {
		if (valor != null)
			return valor.toUpperCase();
		return "";
	}
}