package com.ec.mutualistapichincha.listasnegras.process;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.enterprise.context.ApplicationScoped;

import org.jbpm.services.task.utils.ContentMarshallerHelper;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.model.Content;
import org.kie.api.task.model.Task;
import org.kie.api.task.model.TaskSummary;
import org.kie.services.client.api.RemoteRestRuntimeFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ec.mutualistapichincha.base.model.GlParametros;
import com.ec.mutualistapichincha.base.service.ServicioParametros;
import com.ec.mutualistapichincha.listasnegras.enumerations.BasesNegativas;
import com.ec.mutualistapichincha.listasnegras.enumerations.ParametrosIngresoExcepcion;
import com.ec.mutualistapichincha.listasnegras.exception.ProcessException;
import com.ec.mutualistapichincha.listasnegras.model.Tarea;

@BPMS
@ApplicationScoped
public class GestorProcesosBpms implements GestorProcesos {

	private RuntimeEngine engine;

	private KieSession kieSession;

	private static final String PROCESS_ID = "ListasNegras.ExcepcionListasNegras";

	private static final Logger logger = LoggerFactory.getLogger(GestorProcesos.class);

	@EJB(lookup = "java:global/mupi-base-ear/mupi-base-ejb/ServicioParametrosBean!com.ec.mutualistapichincha.base.service.ServicioParametros")
	private ServicioParametros servicioParametros;

	@PostConstruct
	protected void init() throws Exception {
		logger.info("---- inicializando GestorProcesosBpms -----");
		try {
			if (engine == null) {
				GlParametros pServer = servicioParametros.busquedaParametros("BPMS_SERVER_NAME");
				GlParametros pPort = servicioParametros.busquedaParametros("BPMS_SERVER_PORT");
				GlParametros pUsername = servicioParametros.busquedaParametros("BPMS_SERVER_USER");
				GlParametros pPassword = servicioParametros.busquedaParametros("BPMS_SERVER_PWD");

				URL baseUrl = new URL("http://" + pServer.getValor().trim() + ":" + pPort.getValor().trim() + "/business-central/");
				logger.debug("==== url bpms server: " + baseUrl.toString());
				RemoteRestRuntimeFactory factory = new RemoteRestRuntimeFactory("com.ec.mutualistapichincha:bpm-listas-negras:1.0.0",
						baseUrl, pUsername.getValor().trim(), pPassword.getValor().trim());
				engine = factory.newRuntimeEngine();
				kieSession = engine.getKieSession();
			}
		} catch (MalformedURLException e) {
			throw new Exception(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ec.mutualistapichincha.listasnegras.process.GestorProcesos#
	 * iniciarProcesoIngresoExcepcion(java.util.Map)
	 */
	@Override
	public Number iniciarProcesoIngresoExcepcion(final Map<ParametrosIngresoExcepcion, String> parametros) throws ProcessException {
		Map<String, Object> processParams = new HashMap<String, Object>();

		for (Entry<ParametrosIngresoExcepcion, String> parametro : parametros.entrySet()) {
			if (parametro.getKey().equals(ParametrosIngresoExcepcion.BASES)) {
				validarBases(parametro.getValue());
			}

			processParams.put(parametro.getKey().getNombre(), parametro.getValue());
		}

		ProcessInstance instance = kieSession.startProcess(PROCESS_ID, processParams);
		return instance.getId();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ec.mutualistapichincha.listasnegras.process.GestorProcesos#buscarTareas
	 * (java.lang.String)
	 */
	@Override
	public List<Tarea> buscarTareas(final String usuario) throws ProcessException {
		List<Tarea> tareas = new ArrayList<Tarea>();
		TaskService service = engine.getTaskService();
		List<TaskSummary> tasks = service.getTasksAssignedAsPotentialOwner(usuario, "en-UK");
		for (TaskSummary task : tasks) {
			tareas.add(convertirTarea(task));
		}
		return tareas;
	}

	@Override
	public void reclamarTarea(final Number idTarea, final String usuario, final boolean iniciar) throws ProcessException {
		TaskService service = engine.getTaskService();
		if (iniciar) {
			iniciarTarea(idTarea, usuario);
		} else {
			service.claim(idTarea.longValue(), usuario);
		}
	}

	@Override
	public void iniciarTarea(final Number idTarea, final String usuario) throws ProcessException {
		TaskService service = engine.getTaskService();
		service.start(idTarea.longValue(), usuario);
	}

	@Override
	public Map<?, ?> obtenerVariablesTarea(final Number idTarea) throws ProcessException {
		TaskService service = engine.getTaskService();
		Task task = service.getTaskById(idTarea.longValue());
		long contentId = task.getTaskData().getDocumentContentId();
		Content content = service.getContentById(contentId);
		return ((Map<?, ?>) ContentMarshallerHelper.unmarshall(content.getContent(), null));
	}

	@Override
	public void completarTarea(final Tarea tarea, final String usuario) throws ProcessException {
		TaskService service = engine.getTaskService();

		Map<String, Object> completeVariables = new HashMap<String, Object>();
		for (Entry<String, String> variable : tarea.getVariables().entrySet()) {
			completeVariables.put(variable.getKey(), variable.getKey());
		}

		service.complete(tarea.getIdTarea().longValue(), usuario, completeVariables);
	}

	@Override
	public void abortarProceso(final Number idInstanciaProceso) throws ProcessException {
		kieSession.abortProcessInstance(idInstanciaProceso.longValue());
	}

	// private methods

	private void validarBases(final String value) throws ProcessException {
		logger.info(" validarBase :: value " + value);
		if (value == null || value.trim().isEmpty()) {
			throw new ProcessException("El valor de bases a afectar no es v\u00E1lido: " + value);
		}
		if (value.contains(",")) {
			String[] bases = value.split(",");
			for (String base : bases) {
				validarNombreBase(base);
			}
		} else {
			validarNombreBase(value);
		}
	}

	private void validarNombreBase(final String base) throws ProcessException {
		for (BasesNegativas baseNegativa : BasesNegativas.values()) {
			if (base.equals(baseNegativa.getEtiqueta())) {
				return;
			}
		}
		throw new ProcessException("El valor de la base " + base + " no es v\u00E1lido");
	}

	private Tarea convertirTarea(final TaskSummary task) {
		Tarea tarea = new Tarea();

		logger.debug(" task " + task);
		tarea.setIdTarea(task.getId());
		tarea.setIdInstanciaProceso(task.getProcessInstanceId());
		tarea.setNombre(task.getName());

		if (task.getActualOwner() != null) {
			tarea.setUsuarioActual(task.getActualOwner().getId());
		}
		if (task.getStatus() != null) {
			tarea.setEstado(task.getStatus().name());
		}

		tarea.setFechaInicio(task.getCreatedOn());

		return tarea;
	}

}