package com.ec.mutualistapichincha.listasnegras.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Stateless;

import com.ec.mutualistapichincha.listasnegras.dao.ClNarcoLavadoDaoBean;
import com.ec.mutualistapichincha.listasnegras.dao.ClPepsDaoBean;
import com.ec.mutualistapichincha.listasnegras.dao.ExcepcionesBasesNegDaoBean;
import com.ec.mutualistapichincha.listasnegras.dao.OfacDaoBean;
import com.ec.mutualistapichincha.listasnegras.dto.RespuestaNombrePEP;
import com.ec.mutualistapichincha.listasnegras.dto.RespuestaValidacionBasesNegativas;
import com.ec.mutualistapichincha.listasnegras.dto.ResultadoBaseNegativa;
import com.ec.mutualistapichincha.listasnegras.model.ExcepcionesBasesNeg;
import com.ec.mutualistapichincha.listasnegras.util.MensajesValidacion;
import com.ec.mutualistapichincha.listasnegras.util.Validaciones;

@Stateless
public class ServicioExcepcionesBean implements ServicioExcepciones {

	@EJB
	private ClNarcoLavadoDaoBean narcoLavadoDaoBean;

	@EJB
	private OfacDaoBean ofacDaoBean;

	@EJB
	private ClPepsDaoBean pepsDaoBean;

	@EJB
	private ExcepcionesBasesNegDaoBean excepcionesDao;

	@Override
	public Collection<ExcepcionesBasesNeg> ingresarExcepcion(final String identificacion, final String tipoIdentificacion,
			final String nombres, final String apellidos, final String bases, final Date fechaFin, final String estado,
			final String observaciones) {
		List<ExcepcionesBasesNeg> excepciones = new ArrayList<ExcepcionesBasesNeg>();

		if (bases.contains(",")) {
			String[] nombresBases = bases.split(",");
			for (String nombreBase : nombresBases) {
				excepciones.add(guardarExcepcion(identificacion, tipoIdentificacion, nombres, apellidos, nombreBase, fechaFin, estado,
						observaciones));
			}
		} else {
			excepciones
					.add(guardarExcepcion(identificacion, tipoIdentificacion, nombres, apellidos, bases, fechaFin, estado, observaciones));
		}

		return excepciones;
	}

	private ExcepcionesBasesNeg guardarExcepcion(final String identificacion, final String tipoIdentificacion, final String nombres,
			final String apellidos, final String base, final Date fechaFin, final String estado, final String observaciones) {

		ExcepcionesBasesNeg excepcion = excepcionesDao.buscarCedula(identificacion, base);

		if (excepcion == null) {
			excepcion = new ExcepcionesBasesNeg();
			excepcion.setIdentificacion(identificacion);
			excepcion.setNombres(nombres.toUpperCase());
			excepcion.setApellidos(apellidos.toUpperCase());
			excepcion.setFecha(new Date());
			excepcion.setFechaFin(fechaFin);
			excepcion.setBase(base);
			excepcion.setEstado(estado);
			excepcion.setObservaciones(observaciones);
			excepcionesDao.crear(excepcion);
		} else {
			excepcion.setNombres(nombres.toUpperCase());
			excepcion.setApellidos(apellidos.toUpperCase());
			excepcion.setFecha(new Date());
			excepcion.setFechaFin(fechaFin);
			excepcion.setObservaciones(observaciones);
		}
		return excepcion;
	}

	@Override
	public RespuestaValidacionBasesNegativas validarBasesNegativas(final String cedula, final String nombres, final String apellidos,
			final Integer operador, final String accion, final String modulo) {

		List<ResultadoBaseNegativa> resultados;
		RespuestaValidacionBasesNegativas respuestaValidacionBasesNegativas = new RespuestaValidacionBasesNegativas();

		if (validarDatosCompletosCliente(cedula, apellidos) && validarDatosCompletosBases(operador, accion, modulo)) {

			int resultado;
			resultados = new ArrayList<ResultadoBaseNegativa>();

			resultado = narcoLavadoDaoBean.narcoFichado(cedula, nombres, apellidos, operador, accion, modulo);
			if (resultado > 0) {
				resultados.add(crearResultado(MensajesValidacion.CLIENTE_CONSEP, false, MensajesValidacion.TIPO_BASE_CONSEP));
			}

			resultado = ofacDaoBean.listaOfacNomb(cedula, nombres, apellidos, operador, accion, modulo);
			if (resultado > 0) {
				resultados.add(crearResultado(MensajesValidacion.CLIENTE_OFAC, false, MensajesValidacion.TIPO_BASE_OFAC));
			}

			Object[] objPep = pepsDaoBean.listaPEPNomb(cedula, nombres, apellidos, operador, accion, modulo);
			resultado = (Integer) objPep[0];
			if (resultado > 0) {
				String mensajePep = armarMensajePeps((String) objPep[1]);
				resultados.add(crearResultado(mensajePep, true, MensajesValidacion.TIPO_BASE_PEP));
			}

			respuestaValidacionBasesNegativas.setOperacionExitosa(true);
			respuestaValidacionBasesNegativas.setMensaje(MensajesValidacion.MSJ_EXITOSO);

			if (!resultados.isEmpty()) {
				respuestaValidacionBasesNegativas.setExisteBasesNegativas(true);
				respuestaValidacionBasesNegativas.setResultadoBaseNegativas(resultados);
			} else {
				respuestaValidacionBasesNegativas.setExisteBasesNegativas(false);
			}
		} else {
			respuestaValidacionBasesNegativas.setOperacionExitosa(false);
			respuestaValidacionBasesNegativas.setMensaje(MensajesValidacion.MSJ_NO_EXISTOSO);
		}

		return respuestaValidacionBasesNegativas;
	}

	@Override
	public RespuestaNombrePEP obtenerListaNombrePep(final String cedula, final String nombres, final String apellidos,
			final Integer operador, final String accion, final String modulo) {

		RespuestaNombrePEP respuestaNombrePEP = new RespuestaNombrePEP();
		Object[] resultado = pepsDaoBean.listaPEPNomb(cedula, nombres, apellidos, operador, accion, modulo);

		if (resultado != null) {
			respuestaNombrePEP.setValorPEP((Integer) resultado[0]);
			respuestaNombrePEP.setCargo((String) resultado[1]);
		}
		return respuestaNombrePEP;
	}

	private String armarMensajePeps(final String cargo) {
		return MensajesValidacion.MSJ_PEP_1 + cargo + MensajesValidacion.MSJ_PEP_2;
	}

	private ResultadoBaseNegativa crearResultado(final String mensaje, final boolean permisivo, final String tipoBase) {
		ResultadoBaseNegativa resultadoBaseNegativa = new ResultadoBaseNegativa();
		resultadoBaseNegativa.setMensaje(mensaje);
		resultadoBaseNegativa.setPermisivo(permisivo);
		resultadoBaseNegativa.setTipoBase(tipoBase);
		return resultadoBaseNegativa;
	}

	private boolean validarDatosCompletosCliente(final String cedula, final String apellido) {

		if (Validaciones.stringNoNuloNoVacio(cedula) && Validaciones.stringNoNuloNoVacio(apellido)) {
			return true;
		}
		return false;
	}

	private boolean validarDatosCompletosBases(final Integer operador, final String accion, final String modulo) {

		if (Validaciones.integerNoNuloNoCero(operador) && Validaciones.stringNoNuloNoVacio(accion)
				&& Validaciones.stringNoNuloNoVacio(modulo)) {
			return true;
		}
		return false;
	}
}
