package com.ec.mutualistapichincha.listasnegras.controller;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.event.ActionEvent;
import javax.inject.Inject;

import org.codehaus.plexus.util.ExceptionUtils;

import com.ec.mutualistapichincha.listasnegras.data.TareasDataManager;
import com.ec.mutualistapichincha.listasnegras.exception.ProcessException;
import com.ec.mutualistapichincha.listasnegras.model.Tarea;
import com.ec.mutualistapichincha.listasnegras.process.BPMS;
import com.ec.mutualistapichincha.listasnegras.process.GestorProcesos;
import com.ec.mutualistapichincha.listasnegras.service.ServicioExcepciones;

@SuppressWarnings("serial")
@ManagedBean
@RequestScoped
public class IngresoExcepcionController extends BaseController {

	@ManagedProperty(value = "#{tareasDataManager}")
	private TareasDataManager tareasDataManager;

	@Inject
	@BPMS
	private GestorProcesos gestorProcesos;

	@EJB
	private ServicioExcepciones servicioExcepciones;

	public void init(final ActionEvent event) {
		try {
			cargarVariablesTarea(getTareasDataManager().getTarea(), gestorProcesos);

			Calendar calendar = GregorianCalendar.getInstance();
			calendar.add(Calendar.DAY_OF_YEAR, 365);
			getTareasDataManager().setFecha(calendar.getTime());
		} catch (ProcessException e) {
			agregarMensajeError("Error al cargar los datos de la tarea: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
	}

	public String completar() {
		String retorno = null;
		Tarea tarea = tareasDataManager.getTarea();
		String usuario = getPrincipal().getUsername();
		try {
			servicioExcepciones.ingresarExcepcion(tarea.getVariables().get("iIdentificacion"),
					tarea.getVariables().get("iTipoIdentificacion"), tarea.getVariables().get("iNombres"),
					tarea.getVariables().get("iApellidos"), tarea.getVariables().get("iBasesAfectadas"), tareasDataManager.getFecha(),
					tareasDataManager.getEstado(), tareasDataManager.getObservacionesExcepcion());

			gestorProcesos.completarTarea(tarea, usuario);
			getTareasDataManager().setTarea(null);
			getTareasDataManager().setObservacionesExcepcion(null);
			getTareasDataManager().setEstado(null);
			getTareasDataManager().setFecha(null);
			retorno = "index.jsf?faces-redirect=true";
			agregarMensajeInfo("La tarea se ha completado correctamente", null);
		} catch (ProcessException e) {
			agregarMensajeError("Ocurrio un error al tratar de completar esta tarea: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		} catch (RuntimeException e) {
			agregarMensajeError("Ocurrio un error al tratar de completar esta tarea: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
		return retorno;
	}

	// getter - setter

	public TareasDataManager getTareasDataManager() {
		return tareasDataManager;
	}

	public void setTareasDataManager(final TareasDataManager tareasDataManager) {
		this.tareasDataManager = tareasDataManager;
	}

}
