package com.ec.mutualistapichincha.listasnegras.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.inject.Inject;

import org.codehaus.plexus.util.ExceptionUtils;

import com.ec.mutualistapichincha.listasnegras.data.InicioProcesoDataManager;
import com.ec.mutualistapichincha.listasnegras.enumerations.BasesNegativas;
import com.ec.mutualistapichincha.listasnegras.enumerations.ParametrosIngresoExcepcion;
import com.ec.mutualistapichincha.listasnegras.exception.ProcessException;
import com.ec.mutualistapichincha.listasnegras.process.BPMS;
import com.ec.mutualistapichincha.listasnegras.process.GestorProcesos;

@SuppressWarnings("serial")
@ManagedBean
@RequestScoped
public class InicioProcesoController extends BaseController {

	@ManagedProperty(value = "#{inicioProcesoDataManager}")
	private InicioProcesoDataManager inicioProcesoDataManager;

	@Inject
	@BPMS
	private GestorProcesos gestorProcesos;

	// listener

	public void iniciarProceso(final ActionEvent event) {
		Map<ParametrosIngresoExcepcion, String> params = new HashMap<ParametrosIngresoExcepcion, String>();
		params.put(ParametrosIngresoExcepcion.IDENTIFICACION, getInicioProcesoDataManager().getIdentificacion());
		params.put(ParametrosIngresoExcepcion.TIPO_IDENTIFICACION, getInicioProcesoDataManager().getTipoIdentificacion());
		params.put(ParametrosIngresoExcepcion.NOMBRES, getInicioProcesoDataManager().getNombres());
		params.put(ParametrosIngresoExcepcion.APELLIDOS, getInicioProcesoDataManager().getApellidos());

		StringBuilder builder = new StringBuilder();

		for (String base : getInicioProcesoDataManager().getBases()) {
			builder.append(base.trim());
			builder.append(",");
		}

		String bases = builder.toString();

		params.put(ParametrosIngresoExcepcion.BASES, bases.substring(0, bases.length() - 1));
		try {
			Number idProceso = gestorProcesos.iniciarProcesoIngresoExcepcion(params);
			getInicioProcesoDataManager().setIdProceso(idProceso);
			agregarMensajeInfo("Proceso iniciado correctamente; el id del proceso iniciado es: " + idProceso, null);
		} catch (ProcessException e) {
			agregarMensajeError("Error al iniciar el proceso: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		} catch (RuntimeException e) {
			agregarMensajeError("Error al iniciar el proceso: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
	}

	public void limpiarDatos(final ActionEvent event) {
		getInicioProcesoDataManager().setIdProceso(null);
		getInicioProcesoDataManager().setIdentificacion(null);
		getInicioProcesoDataManager().setTipoIdentificacion(null);
		getInicioProcesoDataManager().setNombres(null);
		getInicioProcesoDataManager().setApellidos(null);
		getInicioProcesoDataManager().setBases(null);
	}

	// public methods

	public List<SelectItem> getBasesItems() {
		List<SelectItem> items = new ArrayList<SelectItem>();
		for (BasesNegativas base : BasesNegativas.values()) {
			items.add(new SelectItem(base.getEtiqueta(), base.getEtiqueta()));
		}
		return items;
	}

	// getter - setter

	public InicioProcesoDataManager getInicioProcesoDataManager() {
		return inicioProcesoDataManager;
	}

	public void setInicioProcesoDataManager(final InicioProcesoDataManager inicioProcesoDataManager) {
		this.inicioProcesoDataManager = inicioProcesoDataManager;
	}

}