package com.ec.mutualistapichincha.listasnegras.controller;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.event.ActionEvent;
import javax.inject.Inject;

import org.codehaus.plexus.util.ExceptionUtils;

import com.ec.mutualistapichincha.listasnegras.data.TareasDataManager;
import com.ec.mutualistapichincha.listasnegras.exception.ProcessException;
import com.ec.mutualistapichincha.listasnegras.model.Tarea;
import com.ec.mutualistapichincha.listasnegras.process.BPMS;
import com.ec.mutualistapichincha.listasnegras.process.GestorProcesos;

@SuppressWarnings("serial")
@ManagedBean
@RequestScoped
public class TareasController extends BaseController {

	@ManagedProperty(value = "#{tareasDataManager}")
	private TareasDataManager tareasDataManager;

	@Inject
	@BPMS
	private GestorProcesos gestorProcesos;

	public void init(final ActionEvent event) {
		String usuario = getPrincipal().getUsername();
		try {
			getTareasDataManager().setTareas(gestorProcesos.buscarTareas(usuario));
		} catch (ProcessException e) {
			agregarMensajeError("Error al buscar las tareas del usuario: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
	}

	public void reclamarTarea(final Tarea tarea) {
		String usuario = getPrincipal().getUsername();
		try {
			gestorProcesos.reclamarTarea(tarea.getIdTarea(), usuario, false);
			getTareasDataManager().setTareas(gestorProcesos.buscarTareas(usuario));
			getTareasDataManager().setTarea(tarea);
			agregarMensajeInfo("Tarea asignada correctamente", null);
		} catch (ProcessException e) {
			agregarMensajeError("Error al asignar la tarea a este usuario: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
	}

	public String reclamarIniciarTarea(final Tarea tarea) {
		String usuario = getPrincipal().getUsername();
		try {
			gestorProcesos.reclamarTarea(tarea.getIdTarea(), usuario, true);
			getTareasDataManager().setTareas(gestorProcesos.buscarTareas(usuario));
			getTareasDataManager().setTarea(tarea);
			agregarMensajeInfo("Tarea asignada e iniciada correctamente", null);
			return direccionar(tarea);
		} catch (ProcessException e) {
			agregarMensajeError("Error al asignar la tarea a este usuario: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
		return null;
	}

	public String iniciarTarea(final Tarea tarea) {
		String usuario = getPrincipal().getUsername();
		try {
			gestorProcesos.iniciarTarea(tarea.getIdTarea(), usuario);
			getTareasDataManager().setTareas(gestorProcesos.buscarTareas(usuario));
			getTareasDataManager().setTarea(tarea);
			agregarMensajeInfo("Tarea iniciada correctamente", null);
			return direccionar(tarea);
		} catch (ProcessException e) {
			agregarMensajeError("Error al asignar la tarea a este usuario: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
		return null;
	}

	public String verTarea(final Tarea tarea) {
		getTareasDataManager().setTarea(tarea);
		return direccionar(tarea);
	}

	public void eliminarProceso(final Tarea tarea) {
		try {
			gestorProcesos.abortarProceso(tarea.getIdInstanciaProceso());
			agregarMensajeInfo("Solicitud y proceso eliminados correctamente", null);
			init(null);
		} catch (ProcessException e) {
			agregarMensajeError("Error al eliminar la instancia de este proceso: " + e.getMessage(), ExceptionUtils.getStackTrace(e));
		}
	}

	// private methods

	private String direccionar(final Tarea tarea) {
		if ("verificacion de documentos".equalsIgnoreCase(tarea.getNombre())) {
			return "verificacionDocumentos.jsf?faces-redirect=true";
		} else if ("ingreso de excepcion".equalsIgnoreCase(tarea.getNombre())) {
			return "ingresoExcepcion.jsf?faces-redirect=true";
		}
		return null;
	}

	// getter - setter

	public TareasDataManager getTareasDataManager() {
		return tareasDataManager;
	}

	public void setTareasDataManager(final TareasDataManager tareasDataManager) {
		this.tareasDataManager = tareasDataManager;
	}

}
