package com.ec.mutualistapichincha.listasnegras.webservice;

import java.util.HashMap;
import java.util.Map;

import javax.ejb.EJB;
import javax.inject.Inject;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

import org.codehaus.plexus.util.ExceptionUtils;

import com.ec.mutualistapichincha.listasnegras.dto.RespuestaInicioProceso;
import com.ec.mutualistapichincha.listasnegras.dto.RespuestaNombrePEP;
import com.ec.mutualistapichincha.listasnegras.dto.RespuestaValidacionBasesNegativas;
import com.ec.mutualistapichincha.listasnegras.enumerations.BasesNegativas;
import com.ec.mutualistapichincha.listasnegras.enumerations.ParametrosIngresoExcepcion;
import com.ec.mutualistapichincha.listasnegras.exception.ProcessException;
import com.ec.mutualistapichincha.listasnegras.process.BPMS;
import com.ec.mutualistapichincha.listasnegras.process.GestorProcesos;
import com.ec.mutualistapichincha.listasnegras.service.ServicioExcepciones;

@WebService
public class ListaNegraWebService {

	@EJB
	private ServicioExcepciones servicioExcepciones;

	@Inject
	@BPMS
	private GestorProcesos gestorProcesos;

	@WebMethod
	public RespuestaValidacionBasesNegativas validarBasesNegativas(@WebParam(name = "cedula") final String cedula,
			@WebParam(name = "nombre") final String nombres, @WebParam(name = "apellido") final String apellidos,
			@WebParam(name = "operador") final Integer operador, @WebParam(name = "accion") final String accion,
			@WebParam(name = "modulo") final String modulo) {

		return servicioExcepciones.validarBasesNegativas(cedula, nombres, apellidos, operador, accion, modulo);
	}

	@WebMethod
	public RespuestaNombrePEP obtenerListaNombrePep(@WebParam(name = "cedula") final String cedula,
			@WebParam(name = "nombre") final String nombres, @WebParam(name = "apellido") final String apellidos,
			@WebParam(name = "operador") final Integer operador, @WebParam(name = "accion") final String accion,
			@WebParam(name = "modulo") final String modulo) {

		return servicioExcepciones.obtenerListaNombrePep(cedula, nombres, apellidos, operador, accion, modulo);
	}

	@WebMethod
	public RespuestaInicioProceso iniciarProceso(@WebParam(name = "identificacion") final String identificacion,
			@WebParam(name = "tipoIdentificacion") final String tipoIdentificacion, @WebParam(name = "nombres") final String nombres,
			@WebParam(name = "apellidos") final String apellidos, @WebParam(name = "basesAfectadas") final String basesNegativas) {
		RespuestaInicioProceso respuesta = new RespuestaInicioProceso();

		try {
			Map<ParametrosIngresoExcepcion, String> parametros = new HashMap<ParametrosIngresoExcepcion, String>();
			parametros.put(ParametrosIngresoExcepcion.IDENTIFICACION, identificacion);
			parametros.put(ParametrosIngresoExcepcion.TIPO_IDENTIFICACION, tipoIdentificacion);
			parametros.put(ParametrosIngresoExcepcion.NOMBRES, nombres);
			parametros.put(ParametrosIngresoExcepcion.APELLIDOS, apellidos);
			parametros.put(ParametrosIngresoExcepcion.BASES, convertirBasesNegativas(basesNegativas));

			Number idInstanciaProceso = gestorProcesos.iniciarProcesoIngresoExcepcion(parametros);
			respuesta.setExitoso(true);
			respuesta.setIdInstanciaProceso(idInstanciaProceso.longValue());
		} catch (ProcessException e) {
			respuesta.setExitoso(false);
			respuesta.setMensaje(ExceptionUtils.getStackTrace(e));
		} catch (RuntimeException e) {
			respuesta.setExitoso(false);
			respuesta.setMensaje(ExceptionUtils.getStackTrace(e));
		}

		return respuesta;

	}
	
	private String convertirBasesNegativas(String basesNegativas){
		String resultado="";
		 if((basesNegativas.contains("OFAC") || basesNegativas.contains("CONSEP")) && !basesNegativas.contains("PEP")){
			 resultado=BasesNegativas.OFAC.getEtiqueta();
		 }else if((!basesNegativas.contains("OFAC") && !basesNegativas.contains("CONSEP")) && basesNegativas.contains("PEP")){
			 resultado=BasesNegativas.PEP.getEtiqueta();
	     }else if((basesNegativas.contains("OFAC") || basesNegativas.contains("CONSEP")) && basesNegativas.contains("PEP")){
	    	 resultado=BasesNegativas.OFAC.getEtiqueta()+","+BasesNegativas.PEP.getEtiqueta();
	     }
		return resultado;
	}
}